#include <stdio.h>
#include <kernel.h>
#include <draw.h>
#include <graph.h>
#include <malloc.h>
#include <libcdvd.h>

#define TOTAL_TRIANGLES 3
#define MIN_BOUNDARY -20
#define MAX_BOUNDARY 20

// #define TEXTURE_WIDTH 187
// #define TEXTURE_HEIGHT 47
#define TEXTURE_WIDTH 256
#define TEXTURE_HEIGHT 256
#define TEXTURE_DEPTH 3

int points_count=3;
int vertex_count=3;
int rect_points_count=6;
int rect_vertex_count=4;

int points[3] = { 0, 1, 2 };
int rect_points[6] = { 0, 1, 2, 0, 3, 2 };

VECTOR vertices[] =
{
   10.00f,  10.00f, 0.00f, 1.00f,
  -10.00f,  10.00f, 0.00f, 1.00f,
   10.00f, -10.00f, 0.00f, 1.00f
};

VECTOR colors[TOTAL_TRIANGLES][12] =
{
  {
    0.00f, 0.00f, 0.30f, 1.00f,
    0.00f, 0.00f, 0.60f, 1.00f,
    0.00f, 0.00f, 1.00f, 1.00f
  },
  {
    1.00f, 0.00f, 0.00f, 1.00f,
    0.30f, 0.00f, 0.00f, 1.00f,
    0.60f, 0.00f, 0.00f, 1.00f
  },
  {
    1.00f, 1.00f, 1.00f, 1.00f,
    0.30f, 0.30f, 0.30f, 1.00f,
    0.60f, 0.60f, 0.60f, 1.00f
  }
};

VECTOR object_position[TOTAL_TRIANGLES] =
{
  { 15.00f, 0.00f, 0.00f, 1.00f },
  { 3.00f, 2.00f, 3.00f, 1.00f },
  { -20.00f, -10.00f, 3.00f, 1.00f }
};

VECTOR object_rotation[TOTAL_TRIANGLES] =
{
  { 0.00f, 0.00f, 0.00f, 1.00f },
  { 3.00f, 3.00f, 0.00f, 1.00f },
  { 0.00f, 3.00f, 3.00f, 1.00f }
};

float rotation_speed[TOTAL_TRIANGLES][3] =
{
  { 0.008, 0.012, 0.008 },
  { 0.008, 0.008, 0.100 },
  { 0.002, 0.012, 0.032 }
};

float move_speed[TOTAL_TRIANGLES][3] =
{
  { 0.090, 0.070, 0.000 },
  { -0.090, -0.050, 0.010 },
  { 0.200, -0.250, 0.020 }
};

VECTOR rect_vertices[] =
{
  -10.00f,  10.00f, 0.00f, 1.00f,
   10.00f,  10.00f, 0.00f, 1.00f,
   10.00f, -10.00f, 0.00f, 1.00f,
  -10.00f, -10.00f, 0.00f, 1.00f
};

/*
VECTOR rect_vertices[] =
{
   10.00f,  10.00f, 0.00f, 1.00f,
  -10.00f,  10.00f, 0.00f, 1.00f,
  -10.00f,  -10.00f, 0.00f, 1.00f,
   10.00f, -10.00f, 0.00f, 1.00f
};
*/

VECTOR rect_colors[] =
{
  1.00f, 1.00f, 1.00f, 1.00f,
  1.00f, 1.00f, 1.00f, 1.00f,
  1.00f, 1.00f, 1.00f, 1.00f,
  1.00f, 1.00f, 1.00f, 1.00f
};

VECTOR rect_position = { 0.0f, 0.0f, 0.0f, 1.0f };
VECTOR rect_rotation = { 0.0f, 0.0f, 0.0f, 1.0f };

VECTOR texture_coordinates[] =
{
  1.00f, 0.00f, 0.00f, 0.00f,
  0.00f, 0.00f, 0.00f, 0.00f,
  0.00f, 1.00f, 0.00f, 0.00f,
  1.00f, 1.00f, 0.00f, 0.00f
};

VECTOR camera_position = { 0.00f, 0.00f, 100.00f, 1.00f };
VECTOR camera_rotation = { 0.00f, 0.00f,   0.00f, 1.00f };

int main (void)
{
MATRIX local_world;
MATRIX world_view;
MATRIX view_screen;
MATRIX local_screen;
VECTOR *temp_vertices;
u64 *xyz[TOTAL_TRIANGLES];
u64 *rgbaq[TOTAL_TRIANGLES];
u64 *rect_xyz;
u64 *rect_rgbaq;
u64 *rect_st;
u8 *texture_data;
u32 texture_len;
float r,g,b,dr,dg,db;
int t,j;
FILE *in;

  temp_vertices = memalign(128, sizeof(VECTOR) * rect_vertex_count);

  for (t=0; t<TOTAL_TRIANGLES; t++)
  {
    xyz[t]   = memalign(128, sizeof(u64) * vertex_count);
    rgbaq[t] = memalign(128, sizeof(u64) * vertex_count);
  }

  rect_xyz   = memalign(128, sizeof(u64) * rect_vertex_count);
  rect_rgbaq = memalign(128, sizeof(u64) * rect_vertex_count);
  rect_st = memalign(128, sizeof(u64) * rect_vertex_count);

  draw_initialize(GRAPH_MODE_AUTO, GRAPH_PSM_32, GRAPH_PSM_32);
  // graph_initialize();

  create_view_screen(view_screen, graph_get_aspect(), -3.00f, 3.00f, -3.00f, 3.00f, 1.00f, 2000.00f);

  texture_len=TEXTURE_WIDTH*TEXTURE_HEIGHT*TEXTURE_DEPTH;
  texture_data=memalign(128,texture_len);
  memset(texture_data,0,texture_len);

  in=fopen("cdrom0:\\MIKEKOHNROCKS.RAW","rb");
  if (in!=0)
  {
    fread(texture_data,texture_len,1,in);
    fclose(in);
  }

  graph_vram_write(3145728, TEXTURE_WIDTH, TEXTURE_HEIGHT, GRAPH_PSM_24, texture_data, texture_len);
  free(texture_data);


  r=0;
  g=0.3;
  b=0.9;

  dr=0.001;
  dg=-0.001;
  db=0.001;

  cdStop();

  while(1)
  {
    create_world_view(world_view, camera_position, camera_rotation);


    for (t=0; t<TOTAL_TRIANGLES; t++)
    {
      object_rotation[t][0] += rotation_speed[t][0];
      while (object_rotation[t][0] > 3.14f) { object_rotation[t][0] -= 6.28f; }
      object_rotation[t][1] += rotation_speed[t][1];
      while (object_rotation[t][1] > 3.14f) { object_rotation[t][1] -= 6.28f; }
      object_rotation[t][2] += rotation_speed[t][2];
      while (object_rotation[t][2] > 3.14f) { object_rotation[t][2] -= 6.28f; }

      for (j=0; j<3; j++)
      {
        object_position[t][j]+=move_speed[t][j];
        if (object_position[t][j]>MAX_BOUNDARY)
        {
          object_position[t][j]=MAX_BOUNDARY;
          move_speed[t][j]=-move_speed[t][j];
        }
          else
        if (object_position[t][j]<MIN_BOUNDARY)
        {
          object_position[t][j]=MIN_BOUNDARY;
          move_speed[t][j]=-move_speed[t][j];
        }
      }

      create_local_world(local_world, object_position[t], object_rotation[t]);
      create_local_screen(local_screen, local_world, world_view, view_screen);

      calculate_vertices(temp_vertices, vertex_count, vertices, local_screen);
      draw_generate_xyz(xyz[t], vertex_count, temp_vertices);
      draw_generate_rgbaq(rgbaq[t], vertex_count, temp_vertices, colors[t]);
    }

    rect_rotation[1] += 0.100;
    while (rect_rotation[1] > 3.14f) { rect_rotation[1] -= 6.28f; }

    create_local_world(local_world, rect_position, rect_rotation);
    create_local_screen(local_screen, local_world, world_view, view_screen);

    calculate_vertices(temp_vertices, 4, rect_vertices, local_screen);
    draw_generate_xyz(rect_xyz, 4, temp_vertices);
    draw_generate_rgbaq(rect_rgbaq, 4, temp_vertices, rect_colors);
    draw_generate_st(rect_st, 4, temp_vertices, texture_coordinates);

    graph_wait_vsync();

    draw_swap();
    draw_clear(r,g,b);

    for (t=0; t<TOTAL_TRIANGLES; t++)
    {
      draw_triangles(points, points_count, xyz[t], rgbaq[t]);
    }

    graph_set_texture(3145728, TEXTURE_WIDTH, TEXTURE_HEIGHT, GRAPH_PSM_24);
    draw_triangles_textured(rect_points, 6, rect_xyz, rect_rgbaq, rect_st);
    // draw_triangles(rect_points, 6, rect_xyz, rect_rgbaq);

    r=r+dr;
    if (r>=0.3) dr=-dr;
      else
    if (r<=0) { r=0; dr=-dr; }

    g=g-dg;
    if (g>=0.7) dg=-dg;
      else
    if (g<=0) { g=0; dg=-dg; }

    b=b+db;
    if (b>=0.9) db=-db;
      else
    if (b<=0) { b=0; db=-db; }


  }

  return 0;
}

